@@ -4,39 +4,38 @@
 # Distributed under the terms of the Modified BSD License.
 
 import atexit
-from binascii import b2a_hex
-from collections import deque
-from imp import lock_held as import_lock_held
+import io
 import os
 import sys
 import threading
-import warnings
-from weakref import WeakSet
 import traceback
+import warnings
+from binascii import b2a_hex
+from collections import deque
 from io import StringIO, TextIOBase
-import io
+from weakref import WeakSet
 
 import zmq
+
 if zmq.pyzmq_version_info() >= (17, 0):
     from tornado.ioloop import IOLoop
 else:
     # deprecated since pyzmq 17
     from zmq.eventloop.ioloop import IOLoop
-from zmq.eventloop.zmqstream import ZMQStream
 
 from jupyter_client.session import extract_header
+from zmq.eventloop.zmqstream import ZMQStream
 
-
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Globals
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 MASTER = 0
 CHILD = 1
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # IO classes
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 
 class IOPubThread:
@@ -88,8 +87,8 @@ class IOPubThread:
         pipe_in = ctx.socket(zmq.PULL)
         pipe_in.linger = 0
 
-        _uuid = b2a_hex(os.urandom(16)).decode('ascii')
-        iface = self._event_interface = 'inproc://%s' % _uuid
+        _uuid = b2a_hex(os.urandom(16)).decode("ascii")
+        iface = self._event_interface = "inproc://%s" % _uuid
         pipe_in.bind(iface)
         self._event_puller = ZMQStream(pipe_in, self.io_loop)
         self._event_puller.on_recv(self._handle_event)
@@ -122,7 +121,7 @@ class IOPubThread:
         # freeze event count so new writes don't extend the queue
         # while we are processing
         n_events = len(self._events)
-        for i in range(n_events):
+        for _ in range(n_events):
             event_f = self._events.popleft()
             event_f()
 
@@ -139,8 +138,9 @@ class IOPubThread:
         try:
             self._pipe_port = pipe_in.bind_to_random_port("tcp://127.0.0.1")
         except zmq.ZMQError as e:
-            warnings.warn("Couldn't bind IOPub Pipe to 127.0.0.1: %s" % e +
-                "\nsubprocess output will be unavailable."
+            warnings.warn(
+                "Couldn't bind IOPub Pipe to 127.0.0.1: %s" % e
+                + "\nsubprocess output will be unavailable."
             )
             self._pipe_flag = False
             pipe_in.close()
@@ -161,7 +161,7 @@ class IOPubThread:
         # must be new context after fork
         ctx = zmq.Context()
         pipe_out = ctx.socket(zmq.PUSH)
-        pipe_out.linger = 3000 # 3s timeout for pipe_out sends before discarding the message
+        pipe_out.linger = 3000  # 3s timeout for pipe_out sends before discarding the message
         pipe_out.connect("tcp://127.0.0.1:%i" % self._pipe_port)
         return ctx, pipe_out
 
@@ -213,7 +213,7 @@ class IOPubThread:
         if self.thread.is_alive():
             self._events.append(f)
             # wake event thread (message content is ignored)
-            self._event_pipe.send(b'')
+            self._event_pipe.send(b"")
         else:
             f()
 
@@ -222,10 +222,13 @@ class IOPubThread:
 
         If my thread isn't running (e.g. forked process), send immediately.
         """
-        self.schedule(lambda : self._really_send(*args, **kwargs))
+        self.schedule(lambda: self._really_send(*args, **kwargs))
 
     def _really_send(self, msg, *args, **kwargs):
         """The callback that actually sends messages"""
+        if self.closed:
+            return
+
         mp_mode = self._check_mp_mode()
 
         if mp_mode != CHILD:
@@ -243,6 +246,7 @@ class IOPubThread:
 
 class BackgroundSocket:
     """Wrapper around IOPub thread that provides zmq send[_multipart]"""
+
     io_thread = None
 
     def __init__(self, io_thread):
@@ -250,7 +254,7 @@ class BackgroundSocket:
 
     def __getattr__(self, attr):
         """Wrap socket attr access for backward-compatibility"""
-        if attr.startswith('__') and attr.endswith('__'):
+        if attr.startswith("__") and attr.endswith("__"):
             # don't wrap magic methods
             super().__getattr__(attr)
         if hasattr(self.io_thread.socket, attr):
@@ -265,7 +269,7 @@ class BackgroundSocket:
         super().__getattr__(attr)
 
     def __setattr__(self, attr, value):
-        if attr == 'io_thread' or (attr.startswith('__' and attr.endswith('__'))):
+        if attr == "io_thread" or (attr.startswith("__" and attr.endswith("__"))):
             super().__setattr__(attr, value)
         else:
             warnings.warn(
@@ -297,8 +301,7 @@ class OutStream(TextIOBase):
     # The time interval between automatic flushes, in seconds.
     flush_interval = 0.2
     topic = None
-    encoding = 'UTF-8'
-
+    encoding = "UTF-8"
 
     def fileno(self):
         """
@@ -332,7 +335,15 @@ class OutStream(TextIOBase):
             self._exc = sys.exc_info()
 
     def __init__(
-        self, session, pub_thread, name, pipe=None, echo=None, *, watchfd=True, isatty=False,
+        self,
+        session,
+        pub_thread,
+        name,
+        pipe=None,
+        echo=None,
+        *,
+        watchfd=True,
+        isatty=False,
     ):
         """
         Parameters
@@ -376,7 +387,8 @@ class OutStream(TextIOBase):
         self._flush_pending = False
         self._subprocess_flush_pending = False
         self._io_loop = pub_thread.io_loop
-        self._new_buffer()
+        self._buffer_lock = threading.RLock()
+        self._buffer = StringIO()
         self.echo = None
         self._isatty = bool(isatty)
 
@@ -391,7 +403,7 @@ class OutStream(TextIOBase):
             self._setup_stream_redirects(name)
 
         if echo:
-            if hasattr(echo, 'read') and hasattr(echo, 'write'):
+            if hasattr(echo, "read") and hasattr(echo, "write"):
                 self.echo = echo
             else:
                 raise ValueError("echo argument must be a file like object")
@@ -448,6 +460,7 @@ class OutStream(TextIOBase):
         # add_timeout has to be handed to the io thread via event pipe
         def _schedule_in_thread():
             self._io_loop.call_later(self.flush_interval, self._flush)
+
         self.pub_thread.schedule(_schedule_in_thread)
 
     def flush(self):
@@ -455,20 +468,22 @@ class OutStream(TextIOBase):
 
         send will happen in the background thread
         """
-        if self.pub_thread and self.pub_thread.thread is not None and self.pub_thread.thread.is_alive():
+        if (
+            self.pub_thread
+            and self.pub_thread.thread is not None
+            and self.pub_thread.thread.is_alive()
+            and self.pub_thread.thread.ident != threading.current_thread().ident
+        ):
             # request flush on the background thread
             self.pub_thread.schedule(self._flush)
             # wait for flush to actually get through, if we can.
-            # waiting across threads during import can cause deadlocks
-            # so only wait if import lock is not held
-            if not import_lock_held():
-                evt = threading.Event()
-                self.pub_thread.schedule(evt.set)
-                # and give a timeout to avoid
-                if not evt.wait(self.flush_timeout):
-                    # write directly to __stderr__ instead of warning because
-                    # if this is happening sys.stderr may be the problem.
-                    print("IOStream.flush timed out", file=sys.__stderr__)
+            evt = threading.Event()
+            self.pub_thread.schedule(evt.set)
+            # and give a timeout to avoid
+            if not evt.wait(self.flush_timeout):
+                # write directly to __stderr__ instead of warning because
+                # if this is happening sys.stderr may be the problem.
+                print("IOStream.flush timed out", file=sys.__stderr__)
         else:
             self._flush()
 
@@ -486,8 +501,7 @@ class OutStream(TextIOBase):
                 self.echo.flush()
             except OSError as e:
                 if self.echo is not sys.__stderr__:
-                    print(f"Flush failed: {e}",
-                          file=sys.__stderr__)
+                    print(f"Flush failed: {e}", file=sys.__stderr__)
 
         data = self._flush_buffer()
         if data:
@@ -495,9 +509,14 @@ class OutStream(TextIOBase):
             # since pub_thread is itself fork-safe.
             # There should be a better way to do this.
             self.session.pid = os.getpid()
-            content = {'name':self.name, 'text':data}
-            self.session.send(self.pub_thread, 'stream', content=content,
-                parent=self.parent_header, ident=self.topic)
+            content = {"name": self.name, "text": data}
+            self.session.send(
+                self.pub_thread,
+                "stream",
+                content=content,
+                parent=self.parent_header,
+                ident=self.topic,
+            )
 
     def write(self, string: str) -> int:
         """Write to current stream after encoding if necessary
@@ -510,25 +529,23 @@ class OutStream(TextIOBase):
         """
 
         if not isinstance(string, str):
-            raise TypeError(
-                f"write() argument must be str, not {type(string)}"
-            )
+            raise TypeError(f"write() argument must be str, not {type(string)}")
 
         if self.echo is not None:
             try:
                 self.echo.write(string)
             except OSError as e:
                 if self.echo is not sys.__stderr__:
-                    print(f"Write failed: {e}",
-                          file=sys.__stderr__)
+                    print(f"Write failed: {e}", file=sys.__stderr__)
 
         if self.pub_thread is None:
-            raise ValueError('I/O operation on closed file')
+            raise ValueError("I/O operation on closed file")
         else:
 
-            is_child = (not self._is_master_process())
+            is_child = not self._is_master_process()
             # only touch the buffer in the IO thread to avoid races
-            self.pub_thread.schedule(lambda: self._buffer.write(string))
+            with self._buffer_lock:
+                self._buffer.write(string)
             if is_child:
                 # mp.Pool cannot be trusted to flush promptly (or ever),
                 # and this helps.
@@ -544,7 +561,7 @@ class OutStream(TextIOBase):
 
     def writelines(self, sequence):
         if self.pub_thread is None:
-            raise ValueError('I/O operation on closed file')
+            raise ValueError("I/O operation on closed file")
         else:
             for string in sequence:
                 self.write(string)
@@ -553,17 +570,15 @@ class OutStream(TextIOBase):
         return True
 
     def _flush_buffer(self):
-        """clear the current buffer and return the current buffer data.
-
-        This should only be called in the IO thread.
-        """
-        data = ''
-        if self._buffer is not None:
-            buf = self._buffer
-            self._new_buffer()
-            data = buf.getvalue()
-            buf.close()
+        """clear the current buffer and return the current buffer data."""
+        buf = self._rotate_buffer()
+        data = buf.getvalue()
+        buf.close()
         return data
 
-    def _new_buffer(self):
-        self._buffer = StringIO()
+    def _rotate_buffer(self):
+        """Returns the current buffer and replaces it with an empty buffer."""
+        with self._buffer_lock:
+            old_buffer = self._buffer
+            self._buffer = StringIO()
+        return old_buffer
