@@ -3,45 +3,48 @@
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-from binascii import b2a_base64
 import math
+import numbers
 import re
 import types
+from binascii import b2a_base64
 from datetime import datetime
-import numbers
+
 from jupyter_client._version import version_info as jupyter_client_version
 
-next_attr_name = '__next__'
+next_attr_name = "__next__"
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Globals and constants
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 # timestamp formats
 ISO8601 = "%Y-%m-%dT%H:%M:%S.%f"
-ISO8601_PAT=re.compile(r"^(\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2})(\.\d{1,6})?Z?([\+\-]\d{2}:?\d{2})?$")
+ISO8601_PAT = re.compile(
+    r"^(\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2})(\.\d{1,6})?Z?([\+\-]\d{2}:?\d{2})?$"
+)
 
 # holy crap, strptime is not threadsafe.
 # Calling it once at import seems to help.
 datetime.strptime("1", "%d")
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Classes and functions
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 
 # constants for identifying png/jpeg data
-PNG = b'\x89PNG\r\n\x1a\n'
+PNG = b"\x89PNG\r\n\x1a\n"
 # front of PNG base64-encoded
-PNG64 = b'iVBORw0KG'
-JPEG = b'\xff\xd8'
+PNG64 = b"iVBORw0KG"
+JPEG = b"\xff\xd8"
 # front of JPEG base64-encoded
-JPEG64 = b'/9'
+JPEG64 = b"/9"
 # constants for identifying gif data
-GIF_64 = b'R0lGODdh'
-GIF89_64 = b'R0lGODlh'
+GIF_64 = b"R0lGODdh"
+GIF89_64 = b"R0lGODlh"
 # front of PDF base64-encoded
-PDF64 = b'JVBER'
+PDF64 = b"JVBER"
 
 JUPYTER_CLIENT_MAJOR_VERSION = jupyter_client_version[0]
 
@@ -126,10 +129,11 @@ def json_clean(obj):
     if isinstance(obj, bytes):
         # unanmbiguous binary data is base64-encoded
         # (this probably should have happened upstream)
-        return b2a_base64(obj).decode('ascii')
+        return b2a_base64(obj).decode("ascii")
 
     if isinstance(obj, container_to_list) or (
-        hasattr(obj, '__iter__') and hasattr(obj, next_attr_name)):
+        hasattr(obj, "__iter__") and hasattr(obj, next_attr_name)
+    ):
         obj = list(obj)
 
     if isinstance(obj, list):
@@ -142,11 +146,13 @@ def json_clean(obj):
         nkeys = len(obj)
         nkeys_collapsed = len(set(map(str, obj)))
         if nkeys != nkeys_collapsed:
-            raise ValueError('dict cannot be safely converted to JSON: '
-                             'key collision would lead to dropped values')
+            raise ValueError(
+                "dict cannot be safely converted to JSON: "
+                "key collision would lead to dropped values"
+            )
         # If all OK, proceed by making the new dict that will be json-safe
         out = {}
-        for k,v in obj.items():
+        for k, v in obj.items():
             out[str(k)] = json_clean(v)
         return out
     if isinstance(obj, datetime):
